import re

RUBIKA_LINK_PATTERN = re.compile(r'\brubika\.ir\b')
GROUP_LINK_PATTERN = re.compile(r'https://rubika\.ir/joing/[A-Z0-9]+')
USERNAME_PATTERN = re.compile(r'@([a-zA-Z0-9_]{3,32})')

def is_rubika_link(string: str) -> bool:
    return bool(RUBIKA_LINK_PATTERN.search(string))

def is_group_link(string: str) -> bool:
    return bool(GROUP_LINK_PATTERN.search(string))

def is_username(string: str) -> bool:
    return bool(USERNAME_PATTERN.search(string))

def get_rubika_links(string: str) -> list:
    return RUBIKA_LINK_PATTERN.findall(string)

def get_group_links(string: str) -> list:
    return GROUP_LINK_PATTERN.findall(string)

def get_usernames(string: str) -> list:
    return USERNAME_PATTERN.findall(string)

def Bold(text: str) -> str:
    return f'**{text.strip()}**'

def Italic(text: str) -> str:
    return f'__{text.strip()}__'

def Underline(text: str) -> str:
    return f'--{text.strip()}--'

def Strike(text: str) -> str:
    return f'~~{text.strip()}~~'

def Spoiler(text: str) -> str:
    return f'||{text.strip()}||'

def Code(text: str):
    return f'`{text.strip()}`'

def Mention(text: str, object_guid: str) -> str:
    return f'[{text.strip()}]({object_guid.strip()})'

def HyperLink(text: str, link: str) -> str:
    return f'[{text.strip()}]({link.strip()})'


import rubpy

class GetTrendStickerSets:
    async def get_trend_sticker_sets(
            self: "rubpy.Client",
            start_id: str = None,
    ):
        return await self.builder(name='getTrendStickerSets',
                                  input={'start_id': str(start_id)})


class GetLinkFromAppUrl:
    async def get_link_from_app_url(
            self,
            app_url: str,
    ):
        return await self.builder('getLinkFromAppUrl',
                                  input={
                                      'app_url': app_url,
                                  })


import rubpy

class SearchStickers:
    async def search_stickers(
            self: "rubpy.Client",
            search_text: str = '',
            start_id: str = None,
    ):
        return await self.builder('searchStickers',
                                  input={
                                      'search_text': search_text,
                                      'start_id': str(start_id),
                                  })


import rubpy
import warnings
import re


system_versions = {
        'Windows NT 10.0': 'Windows 10',
        'Windows NT 6.2': 'Windows 8',
        'Windows NT 6.1': 'Windows 7',
        'Windows NT 6.0': 'Windows Vista',
        'Windows NT 5.1': 'windows XP',
        'Windows NT 5.0': 'Windows 2000',
        'Mac': 'Mac/iOS',
        'X11': 'UNIX',
        'Linux': 'Linux'
    }


async def get_browser(user_agent, lang_code, app_version, *args, **kwargs):
        device_model = re.search(r'(opera|chrome|safari|firefox|msie'
                                 r'|trident)\/(\d+)', user_agent.lower())
        if not device_model:
            device_model = 'Unknown'
            warnings.warn(f'can not parse user-agent ({user_agent})')

        else:
            device_model = device_model.group(1) + ' ' + device_model.group(2)

        system_version = 'Unknown'
        for key, value in system_versions.items():
            if key in user_agent:
                system_version = value
                break

        # window.navigator.mimeTypes.length (outdated . Defaults to '2')
        device_hash = '2'
        return {
            'token': '',
            'lang_code': lang_code,
            'token_type': 'Web',
            'app_version': f'WB_{app_version}',
            'system_version': system_version,
            'device_model': device_model.title(),
            'device_hash': device_hash + ''.join(re.findall(r'\d+', user_agent))}


class RegisterDevice:
    async def register_device(self: "rubpy.Client"):
        return await self.builder(name='registerDevice',
                                 input=await get_browser(self.user_agent,
                                                         self.lang_code,
                                                         self.DEFAULT_PLATFORM.get('app_version')))


class SearchChatMessages:
    async def search_chat_messages(
            self,
            object_guid: str,
            search_text: str,
            type: str = 'Text',
    ):
        if type not in ('Text', 'Hashtag'):
            raise ValueError('`type` argument can only be in ("text", "Hashtag").')

        return await self.builder('searchChatMessages',
                                  input={
                                      'object_guid': object_guid,
                                      'search_text': search_text,
                                      'type': type,
                                  })


class ChatType:
    """Chat type enumeration used in :obj:`~pyrogram.types.Chat`."""

    PRIVATE = 'Private'
    "Chat is a private chat with a user"

    BOT = 'Bot'
    "Chat is a private chat with a bot"

    GROUP = 'Group'
    "Chat is a basic group"

    CHANNEL = 'Channel'
    "Chat is a channel"

    SERVICE = 'Service'
    "Chat is a service"


from .start import Start
from .connect import Connect
from .disconnect import Disconnect
from .add_handler import AddHandler
from .remove_handler import RemoveHandler
from .run import Run
from .upload import UploadFile
from .download import Download
from .get_updates import GetUpdates
from .download_profile_picture import DownloadProfilePicture
from .get_members import GetMembers


class Utilities(
    Start,
    Connect,
    Disconnect,
    AddHandler,
    RemoveHandler,
    Run,
    UploadFile,
    Download,
    GetUpdates,
    DownloadProfilePicture,
    GetMembers,
):
    pass


from datetime import datetime
from .location import Location
from typing import Literal
from pydantic import BaseModel


class LiveLocation(BaseModel):
    start_time: datetime
    live_period: int  # In Second
    current_location: Location
    user_id: str
    status: Literal['Stopped', 'Live']
    last_update_time: datetime


import rubpy

class TerminateSession:
    async def terminate_session(self: "rubpy.Client",
                                session_key: str,
    ):
        return await self.builder('terminateSession',
                                  input={'session_key': session_key})


import rubpy


class GetPollStatus:
    async def get_poll_status(
            self: "rubpy.Client",
            poll_id: str,
    ):
        return self.builder(name='getPollStatus',
                            input={
                                'poll_id': poll_id,
                            })


from .chat_action import ChatAction
from .chat_type import ChatType
from .message_media_type import MessageMediaType
from .poll_type import PollType
from .reaction_type import ReactionType
from .chat_access_type import Access


from .setup_two_step_verification import SetupTwoStepVerification
from .get_two_passcode_status import GetTwoPasscodeStatus
from .get_privacy_setting import GetPrivacySetting
from .get_blocked_users import GetBlockedUsers
from .terminate_session import TerminateSession
from .get_my_sessions import GetMySessions
from .delete_folder import DeleteFolder
from .get_folders import GetFolders
from .get_suggested_folders import GetSuggestedFolders
from .set_setting import SetSetting
from .update_profile import UpdateProfile
from .update_username import UpdateUsername


class Settings(
    SetupTwoStepVerification,
    GetTwoPasscodeStatus,
    GetPrivacySetting,
    GetBlockedUsers,
    TerminateSession,
    GetMySessions,
    DeleteFolder,
    GetFolders,
    GetSuggestedFolders,
    SetSetting,
    UpdateProfile,
    UpdateUsername,
):
    pass


import json
import base64


class StringSession(object):
    def __init__(self, session: str = None) -> None:
        self.session = self.load(session)

    @classmethod
    def load(cls, session):
        if isinstance(session, str):
            return json.loads(base64.b64decode(session))

    @classmethod
    def dump(cls, session):
        if isinstance(session, list):
            session = json.dumps(session).encode('utf-8')
            return base64.b64encode(session).decode('utf-8')

    @classmethod
    def from_sqlite(cls, session):
        session = cls.dump(session.information())
        return StringSession(session)

    def insert(self, phone_number, auth, guid, user_agent, *args, **kwargs):
        self.session = [phone_number, auth, guid, user_agent]

    def information(self):
        return self.session

    def save(self, file_name=None):
        result = self.dump(self.session)
        if result is None:
            if isinstance(file_name, str):
                if not file_name.endswith('.txt'):
                    file_name += '.txt'
                with open(file_name, 'w+') as file:
                    file.write(result)
        return result



from typing import Optional
from pydantic import BaseModel


class ForwardedFrom(BaseModel):
    type_from: Optional[str] = None
    message_id: Optional[str] = None
    object_guid: Optional[str] = None


import rubpy
from ...types.in_chat_members import InChatMembers


class GetMembers:
    async def get_members(
            self: "rubpy.Client",
            object_guid: str,
            start_id: int = None,
            search_text: str = '',
    ) -> InChatMembers:
        if object_guid.startswith('c0'):
            return await self.get_channel_all_members(
                channel_guid=object_guid,
                search_text=search_text,
                start_id=start_id,
            )

        else:
            return await self.get_group_all_members(
                group_guid=object_guid,
                search_text=None if search_text == '' else search_text,
                start_id=start_id,
            )


import rubpy


class AddToMyGifSet:
    async def add_to_my_gif_set(
            self: "rubpy.Client",
            object_guid: str,
            message_id: str,
    ):
        input = dict(
            object_guid=object_guid,
            message_id=message_id,
        )

        return await self.builder('addToMyGifSet', input=input)


from time import time
from typing import Optional, Union
import rubpy

class GetContactsUpdates:
    async def get_contacts_updates(
            self: "rubpy.Client",
            state: Optional[Union[str, int]] = round(time()) - 150,
    ):
        return await self.builder(name='getContactsUpdates',
                                  input={'state': int(state)})


import rubpy
from typing import Union


class GetTranscription:
    async def get_transcription(
            self: "rubpy.Client",
            message_id: Union[str, int],
            transcription_id: str,
    ):
        data = dict(
            message_id=int(message_id),
            transcription_id=transcription_id,
        )

        return await self.builder(
            name='getTranscription',
            input=data,
        )


import rubpy

class GetStickerSetByID:
    async def get_sticker_set_by_id(
            self: "rubpy.Client",
            sticker_set_id: str,
    ):
        return await self.builder(
            name='getStickerSetByID',
            input={'sticker_set_id': str(sticker_set_id)}
        )


from typing import Union


class AddGroup:
    async def add_group(
            self,
            title: str,
            member_guids: Union[str, list],
    ):
        if isinstance(member_guids, str):
            member_guids = list(member_guids)

        return await self.builder('addGroup',
                                  input={
                                      'title': title,
                                      'member_guids': member_guids,
                                  })


from .get_user_info import GetUserInfo
from .get_me import GetMe
from .set_block_user import SetBlockUser
from .delete_user_chat import DeleteUserChat
from .check_user_username import CheckUserUsername


class Users(
    GetUserInfo,
    GetMe,
    SetBlockUser,
    DeleteUserChat,
    CheckUserUsername,
):
    pass


from typing import Optional
import rubpy


class SetSetting:
    async def set_setting(
            self: "rubpy.Client",
            show_my_last_online: str = None,
            show_my_phone_number: str = None,
            show_my_profile_photo: str = None,
            link_forward_message: str = None,
            can_join_chat_by: str = None,
    ):
        input = {}
        updated_parameters = []

        if isinstance(show_my_last_online, str):
            if show_my_last_online not in ('Nobody', 'Everybody', 'MyContacts'):
                raise ValueError('The `show_my_last_online` can only be in `["Nobody", "Everybody", "MyContacts"]`.')

            input['show_my_last_online'] = show_my_last_online
            updated_parameters.append('show_my_last_online')

        if isinstance(show_my_phone_number, str):
            if show_my_phone_number not in ('Nobody', 'Everybody', 'MyContacts'):
                raise ValueError('The `show_my_phone_number` can only be in `["Nobody", "Everybody", "MyContacts"]`.')

            input['show_my_phone_number'] = show_my_phone_number
            updated_parameters.append('show_my_phone_number')

        if isinstance(show_my_profile_photo, str):
            if show_my_profile_photo not in ('Everybody', 'MyContacts'):
                raise ValueError('The `show_my_profile_photo` can only be in `["Everybody", "MyContacts"]`.')

            input['show_my_profile_photo'] = show_my_profile_photo
            updated_parameters.append('show_my_profile_photo')

        if isinstance(link_forward_message, str):
            if link_forward_message not in ('Nobody', 'Everybody', 'MyContacts'):
                raise ValueError('The `link_forward_message` can only be in `["Nobody", "Everybody", "MyContacts"]`.')

            input['link_forward_message'] = link_forward_message
            updated_parameters.append('link_forward_message')

        if isinstance(can_join_chat_by, str):
            if can_join_chat_by not in ('Everybody', 'MyContacts'):
                raise ValueError('The `can_join_chat_by` can only be in `["Everybody", "MyContacts"]`.')

            input['can_join_chat_by'] = can_join_chat_by
            updated_parameters.append('can_join_chat_by')

        input['updated_parameters'] = updated_parameters

        return await self.builder(name='SetSetting', input=input)


class ChatAction:
    """Chat action enumeration used in :obj:`~pyrogram.types.ChatEvent`."""

    TYPING = 'Typing'
    "Typing text message"

    UPLOADING = 'Uploading'
    "Uploading photo"

    RECORD_AUDIO = 'Recording'
    "Recording audio"


import rubpy
from typing import Literal


class JoinChannelAction:
    async def join_channel_action(
            self: "rubpy.Client",
            channel_guid: str,
            action: Literal['Join', 'Remove', 'Archive'],
    ):
        if action not in ['Join', 'Remove', 'Archive']:
            raise ValueError('The `action` argument can only be in `["Join", "Remove", "Archive"]`.')

        return await self.builder(
            name='joinChannelAction',
            input=dict(
                channel_guid=channel_guid,
                action=action,
                ))


import rubpy
import aiofiles

class Download:
    async def download(self: "rubpy.Client", file_inline: "rubpy.types.Results", save_as: str = None, chunk_size: int = 131072, callback=None, *args, **kwargs):
        result = await self.connection.download(
            file_inline.dc_id,
            file_inline.file_id,
            file_inline.access_hash_rec,
            file_inline.size,
            chunk=chunk_size,
            callback=callback)

        if isinstance(save_as, str):
            async with aiofiles.open(save_as, 'wb+') as file:
                await file.write(result)
                return save_as

        return result


class MessageMediaType:
    """Message media type"""

    AUDIO = 'Music'
    "Music media"

    MUSIC = 'Music'
    "Music media"

    DOCUMENT = 'File'
    "Document media"

    PHOTO = 'Image'
    "Photo media"

    STICKER = 'Sticker'
    "Sticker media"

    VIDEO = 'Video'
    "Video media"

    VOICE = 'Voice'
    "Voice media"

    CONTACT = 'Contact'
    "Contact media"

    LOCATION = 'Location'
    "Location media"

    POLL = 'Poll'
    "Poll media"


import sys
import difflib
import inspect
import warnings
import asyncio
from .types import SocketResults


__handlers__ = [
    'ChatUpdates',
    'MessageUpdates',
    'ShowActivities',
    'ShowNotifications',
    'RemoveNotifications'
]

def create(name, __base, authorise: list = [], exception: bool = True, *args, **kwargs):
        result = None
        if name in authorise:
            result = name

        else:
            proposal = difflib.get_close_matches(name, authorise, n=1)
            if proposal:
                result = proposal[0]
                caller = inspect.getframeinfo(inspect.stack()[2][0])
                warnings.warn(
                    f'{caller.filename}:{caller.lineno}: do you mean'
                    f' "{name}", "{result}"? correct it')

        if result is not None or not exception:
            if result is None:
                result = name
            return type(result, __base, {'__name__': result, **kwargs})

        raise AttributeError(f'module has no attribute ({name})')


class BaseHandlers(SocketResults):
    __name__ = 'CustomHandlers'

    def __init__(self, *models, __any: bool = False, **kwargs) -> None:
        self.__models = models
        self.__any = __any

    def is_async(self, value, *args, **kwargs):
        result = False
        if asyncio.iscoroutinefunction(value):
            result = True

        elif asyncio.iscoroutinefunction(value.__call__):
            result = True

        return result

    async def __call__(self, update: dict, *args, **kwargs) -> bool:
        self.original_update = update
        if self.__models:
            for filter in self.__models:
                if callable(filter):
                    # if BaseModels is not called
                    if isinstance(filter, type):
                        filter = filter(func=None)

                    if self.is_async(filter):
                        status = await filter(self, result=None)

                    else:
                        status = filter(self, result=None)

                    if status and self.__any:
                        return True

                    elif not status:
                        return False

        return True


class Handlers:
    def __init__(self, name, *args, **kwargs) -> None:
        self.__name__ = name

    def __eq__(self, value: object) -> bool:
        return BaseHandlers in value.__bases__

    def __dir__(self):
        return sorted(__handlers__)

    def __call__(self, name, *args, **kwargs):
        return self.__getattr__(name)(*args, **kwargs)

    def __getattr__(self, name):
        return create(name, (BaseHandlers,), __handlers__)

sys.modules[__name__] = Handlers(__name__)



from typing import Union


class VotePoll:
    async def vote_poll(
            self,
            poll_id: str,
            selection_index: Union[str, int],
    ):
        return await self.builder('votePoll',
                                  input={
                                      'poll_id': poll_id,
                                      'selection_index': int(selection_index),
                                  })


from typing import Union


class SetGroupDefaultAccess:
    async def set_group_default_access(
            self,
            group_guid: str,
            access_list: Union[str, list],
    ):
        if isinstance(access_list, str):
            access_list = [access_list]

        return await self.builder('setGroupDefaultAccess',
                                  input={
                                      'group_guid': group_guid,
                                      'access_list': access_list,
                                  })


# import os
import sqlite3

suffix = '.rp'
rbs_version = 1


class SQLiteSession(object):

    def __init__(self, session: str) -> None:
        self.filename = session
        if not session.endswith(suffix):
            self.filename += suffix

        self._connection = sqlite3.connect(self.filename,
                                           check_same_thread=False)
        cursor = self._connection.cursor()
        cursor.execute('select name from sqlite_master '
                       'where type=? and name=?', ('table', 'version'))
        if cursor.fetchone():
            cursor.execute('select version from version')
            version = cursor.fetchone()[0]
            if rbs_version != version:
                self.upgrade_database(version)

        else:
            cursor.execute(
                'create table version (version integer primary key)')
            cursor.execute('insert into version values (?)', (rbs_version,))
            cursor.execute('create table session (phone text primary key'
                           ', auth text, guid text, agent text, private_key text)')
            self._connection.commit()
        cursor.close()

    def upgrade_database(self, version):
        pass

    def information(self):
        cursor = self._connection.cursor()
        cursor.execute('select * from session')
        result = cursor.fetchone()
        cursor.close()
        return result

    def insert(self, phone_number, auth, guid, user_agent, private_key, *args, **kwargs):
        cursor = self._connection.cursor()
        cursor.execute(
            'insert or replace into session (phone, auth, guid, agent, private_key)'
            ' values (?, ?, ?, ?, ?)',
            (phone_number, auth, guid, user_agent, private_key)
        )
        self._connection.commit()
        cursor.close()

    @classmethod
    def from_string(cls, session, file_name=None):
        info = session.information()
        if file_name is None:
            if info is None:
                raise ValueError('file_name arg is not set')
            file_name = info[0]

        session = SQLiteSession(file_name)
        if info is not None:
            session.insert(*info)

        return session



from pydantic import BaseModel
from typing import Optional
from .online_time import OnlineTime
from .avatar_thumbnail import AvatarThumbnail


class Member(BaseModel):
    member_type: Optional[str] = None
    member_guid: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None
    last_online: Optional[int] = None
    join_type: Optional[str] = None
    username: Optional[str] = None
    online_time: Optional[OnlineTime] = None


from typing import Union

ALLOWEDS = ["SetAdmin", "BanMember", "ChangeInfo", "PinMessages", "SetJoinLink", "SetMemberAccess", "DeleteGlobalAllMessages"]

class SetGroupAdmin:
    async def set_group_admin(
            self,
            group_guid: str,
            member_guid: str,
            action: str = 'SetAdmin',
            access_list: Union[str, list]=[],
    ):
        if action not in ('SetAdmin', 'UnsetAdmin'):
            raise ValueError('`action` argument can only be in `["SetAdmin", "UnsetAdmin"]`.')

        if isinstance(access_list, str):
            access_list = list(access_list)

        return await self.builder('setGroupAdmin',
                                  input={
                                      'group_guid': group_guid,
                                      'member_guid': member_guid,
                                      'action': action,
                                      'access_list': access_list,
                                  })


from typing import Optional
from pydantic import BaseModel
from .sticker import Sticker
from .file_inline import FileInline


class Message(BaseModel):
    message_id: Optional[str] = None
    text: Optional[str] = None
    time: Optional[str] = None
    is_edited: Optional[bool] = None
    type: Optional[str] = None
    author_type: Optional[str] = None
    author_object_guid: Optional[str] = None
    sticker: Optional[Sticker] = None
    file_inline: Optional[FileInline] = None


class GetGroupMentionList:
    async def get_group_mention_list(
            self,
            group_guid: str,
            search_mention: str = None,
    ):
        return await self.builder('getGroupMentionList',
                                  input={
                                      'group_guid': group_guid,
                                      'search_mention': search_mention,
                                  })


class DeleteAvatar:
    async def delete_avatar(
            self,
            object_guid: str,
            avatar_id: str,
    ):
        return await self.builder('deleteAvatar',
                                  input={
                                      'object_guid': object_guid,
                                      'avatar_id': avatar_id,
                                  })


from ...crypto import Crypto
from ... import exceptions
from ...types import Results
from typing import Union
import rubpy

class Builder:
    async def builder(
            self: "rubpy.Client",
            name: str,
            tmp_session: bool = False,
            encrypt: bool = True,
            dict: bool = False,
            input: dict = None,
    ) -> Union[Results, dict]:
        if not self.connection.api_url:
            await self.connection.get_dcs()

        if self.auth is None:
            self.auth = Crypto.secret(length=32)
            # self._client._logger.info(
            #     'create auth secret', extra={'data': self._client._auth})

        if self.key is None:
            self.key = Crypto.passphrase(self.auth)
            # self._client._logger.info(
            #     'create key passphrase', extra={'data': self._client._key})

        result = await self.connection.send(method=name,
                                            tmp_session=tmp_session,
                                            encrypt=encrypt,
                                            input=input)
        data_enc = result.get('data_enc')
        if data_enc:
            result = Crypto.decrypt(data_enc,
                                key=self.key)

        status = result['status']
        status_det = result['status_det']

        if status == 'OK' and status_det == 'OK':
            if dict:
                return result.get('data')

            result['data']['_client'] = self
            return Results(result['data'])

        raise exceptions(status_det)(result, request=None)


from typing import Literal
from pydantic import BaseModel


class Chat(BaseModel):
    chat_id: str
    chat_type: Literal['User', 'Bot', 'Group', 'Channel']
    user_id: str
    first_name: str
    last_name: str
    title: str
    username: str


from typing import Optional
from pydantic import BaseModel
from .online_time import OnlineTime


class User(BaseModel):
    user_guid: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    phone: Optional[str] = None
    username: Optional[str] = None
    last_online: Optional[int] = None
    bio: Optional[str] = None
    is_deleted: Optional[bool] = None
    is_verified: Optional[bool] = None
    online_time: Optional[OnlineTime] = None


class JoinGroup:
    async def join_group(
            self,
            link: str,
    ):
        if '/' in link:
            link = link.split('/')[-1]

        return await self.builder('joinGroup',
                                  input={
                                      'hash_link': link,
                                  })


from setuptools import setup, find_packages

requirements = [
    'aiohttp',
    'pycryptodome',
    'pydantic==1.10.12',
    'aiofiles',
]

with open('README.md', encoding='UTF-8') as f:
    readme = f.read()

setup(
    name='aiorubika',
    version='0.0.2',
    author='amirali irvany',
    author_email='dev.amirali.irvany@gmail.com',
    description=' Rubika API Framework for Python',
    keywords=['rubika', 'aiorubika', 'chat', 'bot', 'robot', 'asyncio'],
    long_description=readme,
    python_requires="~=3.7",
    long_description_content_type='text/markdown',
    url='https://github.com/metect/rubika',
    packages=find_packages(),
    exclude_package_data={'': ['*.pyc', '*__pycache__*']},
    install_requires=requirements,
    extras_require={
        'cv': ['opencv-python'],
        'pil': ['pillow']
    },
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Topic :: Internet',
        'Topic :: Communications',
        'Topic :: Communications :: Chat',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries :: Application Frameworks'
    ],
)



import rubpy


class LeaveChat:
    async def leave_chat(
            self: "rubpy.Client",
            object_guid: str,
    ):
        if object_guid.startswith('c0'):
            return await self.join_channel_action(object_guid, 'Remove')
        elif object_guid.startswith('g0'):
            return await self.leave_group(object_guid)


class SetBlockUser:
    async def set_block_user(
            self,
            user_guid: str,
            action: str = 'Block',
    ):
        if not action in ():
            raise ValueError('`action` argument can only be `Block` or `Unblock`.')

        return await self.builder('setBlockUser',
                                  input={
                                      'user_guid': user_guid,
                                      'action': action,
                                  })


from typing import Union


class DeleteMessages:
    async def delete_messages(
            self,
            object_guid: str,
            message_ids: Union[str, list],
            type: str = 'Global',
    ):
        if type not in ('Global', 'Local'):
            raise ValueError('`type` argument can only be in ("Global", "Local").')

        if isinstance(message_ids, str):
            message_ids = [message_ids]

        return await self.builder('deleteMessages',
                                  input={
                                      'object_guid': object_guid,
                                      'message_ids': message_ids,
                                      'type': type,
                                  })


from typing import Union, Literal


class SetPinMessage:
    async def set_pin_message(
            self,
            object_guid: str,
            message_id: Union[str, int],
            action: str = Literal['Pin', 'Unpin']
    ):
        if action not in ('Pin', 'Unpin'):
            raise ValueError('The `action` argument can only be in `("Pin", "Unpin")`.')

        return await self.builder('setPinMessage',
                                  input={
                                      'object_guid': object_guid,
                                      'message_id': str(message_id),
                                      'action': action,
                                  })

    async def set_pin(
            self,
            object_guid: str,
            message_id: Union[str, int],
    ):
        return await self.set_pin_message(object_guid=object_guid,
                                          message_id=message_id, action='Pin')

    async def set_unpin(
            self,
            object_guid: str,
            message_id: Union[str, int],
    ):
        return await self.set_pin_message(object_guid=object_guid,
                                          message_id=message_id, action='Unpin')


class BanGroupMember:
    async def ban_group_member(
            self,
            group_guid: str,
            member_guid: str,
            action: str = 'Set',
    ):
        if action not in ["Set", "Unset"]:
            raise ValueError('`action` argument can only be in `["Set", "Unset"]`.')

        return await self.builder('banGroupMember',
                                  input={
                                      'group_guid': group_guid,
                                      'member_guid': member_guid,
                                      'action': action,
                                  })


import rubpy

class DiscardChannelVoiceChat:
    async def discard_channel_voice_chat(
            self: "rubpy.Client",
            channel_guid: str,
            voice_chat_id: str,
    ):
        return await self.builder(name='discardChannelVoiceChat',
                                  input={
                                      'channel_guid': channel_guid,
                                      'voice_chat_id': voice_chat_id,
                                  })


from typing import Optional, List
from pydantic import BaseModel
from .message import Message
from .abs_object import AbsObject


class Chat(BaseModel):
    object_guid: Optional[str] = None
    access: Optional[List[str]] = None
    count_unseen: Optional[int] = None
    is_mute: Optional[bool] = None
    is_pinned: Optional[bool] = None
    time_string: Optional[str] = None
    last_message: Optional[Message] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[str] = None
    status: Optional[str] = None
    time: Optional[int] = None
    abs_object: Optional[AbsObject] = None
    is_blocked: Optional[bool] = None
    last_message_id: Optional[str] = None
    last_deleted_mid: Optional[str] = None
    is_in_contact: Optional[bool] = None


import re
import json
import base64
import string
import secrets
from Crypto.Cipher import AES
from Crypto.Hash import SHA256
from Crypto.Signature import pkcs1_15
from Crypto.Util.Padding import pad, unpad
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP
from string import ascii_lowercase, ascii_uppercase

class Crypto:
    AES_IV = b'\x00' * 16

    @staticmethod
    def decode_auth(auth: str) -> str:
        """
        Decode an auth string by applying character substitutions.

        Args:
            auth (str): The input auth string.

        Returns:
            str: The decoded auth string.
        """
        result_list, digits = [], '0123456789'
        translation_table_lower = str.maketrans(
            ascii_lowercase,
            ''.join([chr(((32 - (ord(c) - 97)) % 26) + 97) for c in ascii_lowercase])
        )
        translation_table_upper = str.maketrans(
            ascii_uppercase,
            ''.join([chr(((29 - (ord(c) - 65)) % 26) + 65) for c in ascii_uppercase])
        )

        for char in auth:
            if char in ascii_lowercase:
                result_list.append(char.translate(translation_table_lower))
            elif char in ascii_uppercase:
                result_list.append(char.translate(translation_table_upper))
            elif char in digits:
                result_list.append(chr(((13 - (ord(char) - 48)) % 10) + 48))
            else:
                result_list.append(char)

        return ''.join(result_list)

    @classmethod
    def passphrase(cls, auth):
        """
        Generate a passphrase from an auth string.

        Args:
            auth (str): The input auth string.

        Returns:
            str: The generated passphrase.
        """
        if len(auth) != 32:
            raise ValueError('auth length should be 32 digits')

        result_list = []
        chunks = re.findall(r'\S{8}', auth)
        for character in (chunks[2] + chunks[0] + chunks[3] + chunks[1]):
            result_list.append(chr(((ord(character) - 97 + 9) % 26) + 97))
        return ''.join(result_list)

    @classmethod
    def secret(cls, length):
        """
        Generate a random secret of the given length.

        Args:
            length (int): Length of the secret.

        Returns:
            str: The generated secret.
        """
        return ''.join(secrets.choice(string.ascii_lowercase)
                       for _ in range(length))

    @classmethod
    def decrypt(cls, data, key):
        """
        Decrypt data using AES encryption.

        Args:
            data (str): The encrypted data.
            key (str): The encryption key.

        Returns:
            dict: The decrypted data as a dictionary.
        """
        aes = AES.new(key.encode(), AES.MODE_CBC, cls.AES_IV)
        dec = aes.decrypt(base64.urlsafe_b64decode(data.encode('UTF-8')))
        return json.loads(unpad(dec, AES.block_size).decode('UTF-8'))

    @classmethod
    def encrypt(cls, data: str, key: str):
        """
        Encrypt data using AES encryption.

        Args:
            data (str or dict): The data to be encrypted.
            key (str): The encryption key.

        Returns:
            str: The encrypted data as a string.
        """
        if isinstance(data, dict):
            data = json.dumps(data)
        raw = pad(data.encode('UTF-8'), AES.block_size)
        aes = AES.new(key.encode(), AES.MODE_CBC, cls.AES_IV)
        return base64.b64encode(aes.encrypt(raw)).decode('UTF-8')

    @staticmethod
    def sign(pkcs1_15: "pkcs1_15.new", data: str) -> str:
        """
        Sign data using an RSA private key.

        Args:
            private_key (str): The RSA private key.
            data (str): The data to be signed.

        Returns:
            str: The base64-encoded signature.
        """
        #key = RSA.import_key(private_key.encode('utf-8'))
        signature = pkcs1_15.sign(SHA256.new(data.encode('utf-8')))
        return base64.b64encode(signature).decode('utf-8')

    @staticmethod
    def create_keys() -> tuple:
        """
        Generate RSA public and private keys.

        Returns:
            tuple: A tuple containing the base64-encoded public key and the private key.
        """
        keys = RSA.generate(1024)
        public_key = Crypto.decode_auth(base64.b64encode(keys.publickey().export_key()).decode('utf-8'))
        private_key = keys.export_key().decode('utf-8')
        return public_key, private_key

    @staticmethod
    def decrypt_RSA_OAEP(private_key: str, data: str):
        """
        Decrypt data using RSA OAEP encryption.

        Args:
            private_key (str): The RSA private key.
            data (str): The encrypted data.

        Returns:
            str: The decrypted data as a string.
        """
        key = RSA.import_key(private_key.encode('utf-8'))
        return PKCS1_OAEP.new(key).decrypt(base64.b64decode(data)).decode('utf-8')


class SendChatActivity:
    async def send_chat_activity(
            self,
            object_guid: str,
            activity: str = 'Typing',
    ):
        if activity not in ('Typing', 'Uploading', 'Recording'):
            raise ValueError('`activity` argument can only be in `["Typing", "Uploading", "Recording"]`')
    
        return await self.builder('sendChatActivity',
                                  input={
                                      'object_guid': object_guid,
                                      'activity': activity,
                                  })


class Access:
    ChangeInfo = "ChangeInfo"
    PinMessages = "PinMessages"
    DeleteGlobalAllMessages = "DeleteGlobalAllMessages"
    BanMember = "BanMember"
    SetAdmin = "SetAdmin"
    SetJoinLink = "SetJoinLink"
    SetMemberAccess = "SetMemberAccess"
    ViewMembers = "ViewMembers"
    ViewAdmins = "ViewAdmins"
    SendMessages = "SendMessages"
    AddMember = "AddMember"
    ViewInfo = "ViewInfo"
    ViewMessages = "ViewMessages"
    DeleteLocalMessages = "DeleteLocalMessages"
    EditMyMessages = "EditMyMessages"
    DeleteGlobalMyMessages = "DeleteGlobalMyMessages"


import rubpy

class RemoveReaction:
    async def remove_reaction(
            self: "rubpy.Client",
            object_guid: str,
            message_id: str,
            reaction_id: int,
    ):
        return await self.action_on_message_reaction(
            object_guid=object_guid,
            message_id=message_id,
            action='Remove',
            reaction_id=reaction_id,
        )


import rubpy

class JoinChannelByLink:
    async def join_channel_by_link(
            self: "rubpy.Client",
            link: str,
    ):
        if '/' in link:
            link = link.split('/')[-1]

        return await self.builder('joinChannelByLink',
                                  input={'hash_link': link})


import rubpy

class DeleteFolder:
    async def delete_folder(
            self: "rubpy.Client",
            folder_id: str,
    ):
        return await self.builder(name='deleteFolder',
                                  input={'folder_id': str(folder_id)})


from ...crypto import Crypto
from ...network import Network


class Connect:
    async def connect(self):
        self.connection = Network(client=self)

        if self.auth and self.private_key is not None:
            self.guid = (await self.get_me()).user.user_guid

        information = self.session.information()
        #self.logger.info(f'the session information was read {information}')

        if information:
            self.auth = information[1]
            self.guid = information[2]
            self.private_key = information[4]

            if isinstance(information[3], str):
                self.user_agent = information[3] or self.user_agent

        return self


from time import time


class GetGroupVoiceChatUpdates:
    async def get_group_voice_chat_updates(
            self,
            group_guid: str,
            voice_chat_id: str,
            state: int=None,
    ):
        if state is None:
            state = round(time()) - 150

        return await self.builder('getGroupVoiceChatUpdates',
                                  input={
                                      'group_guid': group_guid,
                                      'voice_chat_id': voice_chat_id,
                                      'state': int(state),
                                  })


from typing import Literal
from pydantic import BaseModel
from .location import Location


class ButtonLocation(BaseModel):
    default_pointer_location: Location
    default_map_location: Location
    type: Literal['Picker', 'View']
    title: str
    location_image_url: str


import rubpy

class DownloadProfilePicture:
    async def download_profile_picture(
            self: "rubpy.Client",
            object_guid: str,
    ):
        object = await self.get_info(object_guid)

        if object_guid.startswith('c0'):
            avatar_thumbnail = object.channel.avatar_thumbnail
        elif object_guid.startswith('g0'):
            avatar_thumbnail = object.group.avatar_thumbnail
        else:
            avatar_thumbnail = object.user.avatar_thumbnail

        if avatar_thumbnail:
            async with self.connection.session.get(
                url=f'https://messenger{avatar_thumbnail.dc_id}.iranlms.ir/InternFile.ashx',
                params={'id': avatar_thumbnail.file_id, 'ach': avatar_thumbnail.access_hash_rec},
            ) as response:
                if response.ok:
                    return await response.read()


from typing import Union
import rubpy

class SetupTwoStepVerification:
    async def setup_two_step_verification(
            self: "rubpy.Client",
            password: Union[int, str],
            hint: str,
            recovery_email: str,
    ):
        return await self.builder(name='setupTwoStepVerification',
                                  input={'password': str(password),
                                         'hint': hint,
                                         'recovery_email': recovery_email})


import rubpy

class LeaveGroup:
    async def leave_group(
            self: "rubpy.Client",
            group_guid: str,
    ):
        return await self.builder('leaveGroup', input=dict(group_guid=group_guid))


import rubpy

class Join:
    async def join_chat(
            self: "rubpy.Client",
            chat: str,
    ):
        if chat.startswith('c0'):
            return await self.join_channel_action(chat, 'Join')
        else:
            if '@' not in chat:
                return await self.join_group(chat)
            else:
                return await self.join_channel_by_link(chat)


import rubpy


class DeleteContact:
    async def delete_contact(
            self: "rubpy.Client",
            user_guid: str,
    ):
        return self.builder(name='deleteContact',
                            input={'user_guid': user_guid})


from .get_my_sticker_sets import GetMyStickerSets
from .action_on_sticker_set import ActionOnStickerSet
from .get_sticker_set_by_id import GetStickerSetByID
from .get_stickers_by_emoji import GetStickersByEmoji
from .get_stickers_by_set_ids import GetStickersBySetIDs
from .get_trend_sticker_sets import GetTrendStickerSets
from .search_stickers import SearchStickers


class Stickers(
    GetMyStickerSets,
    ActionOnStickerSet,
    GetStickerSetByID,
    GetStickersByEmoji,
    GetStickersBySetIDs,
    GetTrendStickerSets,
    SearchStickers,
):
    pass


class LeaveGroup:
    async def leave_group(
            self,
            group_guid: str,
    ):
        return await self.builder('leaveGroup',
                                  input={
                                      'group_guid': group_guid,
                                  })


from typing import Optional
import rubpy

class EditChannelInfo:
    async def edit_channel_info(
            self: "rubpy.Client",
            channel_guid: str,
            title: Optional[str]=None,
            description: Optional[str]=None,
            channel_type: Optional[str]=None,
            sign_messages: Optional[str]=None,
            chat_reaction_setting: Optional[dict]=None,
            chat_history_for_new_members: Optional[str]=None,
    ):
        updated_parameters = []
        input = {
            'channel_guid': channel_guid,
        }

        if title is not None:
            input['title'] = title
            updated_parameters.append('title')

        if description is not None:
            input['description'] = description
            updated_parameters.append('description')

        if channel_type is not None:
            input['channel_type'] = channel_type
            updated_parameters.append('channel_type')

        if sign_messages is not None:
            input['sign_messages'] = sign_messages
            updated_parameters.append('sign_messages')

        if chat_reaction_setting is not None:
            input['chat_reaction_setting'] = chat_reaction_setting
            updated_parameters.append('chat_reaction_setting')

        if chat_history_for_new_members is not None:
            if chat_history_for_new_members not in ('Hidden', 'Visible'):
                raise ValueError('`chat_history_for_new_members` argument can only be in `["Hidden", "Visible"]`.')

            input['chat_history_for_new_members'] = chat_history_for_new_members
            updated_parameters.append('chat_history_for_new_members')

        input['updated_parameters'] = updated_parameters
        return await self.builder('editChannelInfo',
                                  input=input)


from typing import Optional
from pydantic import BaseModel


class AvatarThumbnail(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None


from typing import Union
import rubpy

class GetStickersBySetIDs:
    async def GetStickersBySetIDs(
            self: "rubpy.Client",
            sticker_set_ids: Union[str, list],
    ):
        if isinstance(sticker_set_ids, str):
            sticker_set_ids = [str(sticker_set_ids)]
        
        return await self.builder(name='GetStickersBySetIDs',
                                  input={'sticker_set_ids': sticker_set_ids})


from typing import Union


class EditMessage:
    async def edit_message(
            self,
            object_guid: str,
            message_id: Union[int, str],
            text: str,
    ):
        input = {
            'object_guid': object_guid,
            'message_id': str(message_id),
            'text': text.strip(),
            }

        markdown = self.markdown.to_metadata(text)
        if 'metadata' in markdown.keys():
            input['metadata'] = markdown.get('metadata')
            input['text'] = markdown.get('text')

        return await self.builder('editMessage',
                                  input=input)


from .get_contacts_updates import GetContactsUpdates
from .add_address_book import AddAddressBook
from .delete_contact import DeleteContact
from .get_contacts import GetContacts


class Contacts(
    GetContactsUpdates,
    AddAddressBook,
    DeleteContact,
    GetContacts,
):
    pass


from pydantic import BaseModel, HttpUrl, Field, PositiveInt
from typing import Optional, List

class AvatarThumbnail(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None

class Object(BaseModel):
    object_guid: Optional[str] = None
    type: Optional[str] = None
    title: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None
    count_members: Optional[int] = None
    username: Optional[str] = None

class SearchGlobalObjects(BaseModel):
    objects: List[Object]
    has_continue: Optional[bool] = None
    timestamp: Optional[str] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class OnlineTime(BaseModel):
    type: Optional[str] = None
    approximate_period: Optional[str] = None

class AvatarThumbnail(BaseModel):
    file_id: Optional[str] = None
    mime: Optional[str] = None
    dc_id: Optional[str] = None
    access_hash_rec: Optional[str] = None

class LastMessage(BaseModel):
    message_id: Optional[str] = None
    type: Optional[str] = None
    text: Optional[str] = None
    author_object_guid: Optional[str] = None
    is_mine: Optional[bool] = None
    author_type: Optional[str] = None

class AbsObject(BaseModel):
    object_guid: Optional[str] = None
    type: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    is_verified: Optional[bool] = None
    is_deleted: Optional[bool] = None

class Chat(BaseModel):
    object_guid: Optional[str] = None
    access: Optional[List[str]] = []
    count_unseen: Optional[int] = None
    is_mute: Optional[bool] = None
    is_pinned: Optional[bool] = None
    time_string: Optional[str] = None
    last_message: Optional[LastMessage] = None
    last_seen_my_mid: Optional[str] = None
    last_seen_peer_mid: Optional[str] = None
    status: Optional[str] = None
    time: Optional[int] = None
    abs_object: Optional[AbsObject] = None
    is_blocked: Optional[bool] = None
    last_message_id: Optional[str] = None
    last_deleted_mid: Optional[str] = None
    is_in_contact: Optional[bool] = None

class ChatReactionSetting(BaseModel):
    reaction_type: Optional[str] = Field(None, description="Type of reactions in chat")
    selected_reactions: Optional[List[str]] = Field(None, description="List of selected reactions in the chat")

class User(BaseModel):
    user_guid: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    phone: Optional[str] = None
    username: Optional[str] = None
    avatar_thumbnail: Optional[AvatarThumbnail] = None
    last_online: Optional[int] = None
    bio: Optional[str] = None
    is_deleted: Optional[bool] = None
    is_verified: Optional[bool] = None
    online_time: Optional[OnlineTime] = None

class Channel(BaseModel):
    channel_guid: Optional[str] = Field(None, description="Channel ID")
    channel_title: Optional[str] = Field(None, description="Channel title")
    avatar_thumbnail: Optional[AvatarThumbnail] = Field(None, description="Profile image information")
    count_members: Optional[PositiveInt] = Field(None, description="Number of channel members")
    description: Optional[str] = Field(None, description="Channel description")
    username: Optional[str] = Field(None, description="Channel username")
    is_deleted: Optional[bool] = Field(None, description="Channel deletion status")
    is_verified: Optional[bool] = Field(None, description="Channel verification status")
    share_url: Optional[HttpUrl] = Field(None, description="Channel sharing URL")
    channel_type: Optional[str] = Field(None, description="Channel type")
    sign_messages: Optional[bool] = Field(None, description="Message signing capability status")
    chat_reaction_setting: Optional[ChatReactionSetting] = Field(None, description="Conversation reaction settings")

class GetObjectByUsername(BaseModel):
    exist: Optional[bool] = None
    type: Optional[str] = None
    user: Optional[User] = None
    channel: Optional[Channel] = None
    chat: Optional[Chat] = None
    timestamp: Optional[str] = None
    is_in_contact: Optional[bool] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None

class OpenChatData(BaseModel):
    object_guid: Optional[str] = None
    object_type: Optional[str] = None
    message_id: Optional[int] = None

class LinkData(BaseModel):
    type: Optional[str] = None
    link_url: Optional[str] = None
    open_chat_data: Optional[OpenChatData] = None

class GetLinkFromAppUrl(BaseModel):
    link: Optional[LinkData] = None

class ChatGuidType(BaseModel):
    type: Optional[str] = None
    object_guid: Optional[str] = None

class MessageInfo(BaseModel):
    message_id: Optional[str] = None
    type: Optional[str] = None
    text: Optional[str] = None
    is_mine: Optional[bool] = None

class ChatInfo(BaseModel):
    time_string: Optional[str] = None
    last_message: Optional[MessageInfo] = None
    time: Optional[int] = None
    last_message_id: Optional[str] = None
    group_voice_chat_id: Optional[str] = None

class ChatUpdate(BaseModel):
    object_guid: Optional[str] = None
    action: Optional[str] = None
    chat: Optional[ChatInfo] = None
    updated_parameters: Optional[List[str]] = None
    timestamp: Optional[str] = None
    type: Optional[str] = None

class PerformerData(BaseModel):
    type: Optional[str] = None
    object_guid: Optional[str] = None

class EventData(BaseModel):
    type: Optional[str] = None
    performer_object: Optional[PerformerData] = None

class MessageData(BaseModel):
    message_id: Optional[str] = None
    text: Optional[str] = None
    time: Optional[str] = None
    is_edited: Optional[bool] = None
    type: Optional[str] = None
    event_data: Optional[EventData] = None

class MessageUpdate(BaseModel):
    message_id: Optional[str] = None
    action: Optional[str] = None
    message: Optional[MessageData] = None
    updated_parameters: Optional[List[str]] = None
    timestamp: Optional[str] = None
    prev_message_id: Optional[str] = None
    object_guid: Optional[str] = None
    type: Optional[str] = None
    state: Optional[str] = None

class GroupVoiceChatData(BaseModel):
    voice_chat_id: Optional[str] = None
    state: Optional[str] = None
    join_muted: Optional[bool] = None
    participant_count: Optional[int] = None
    title: Optional[str] = None
    version: Optional[int] = None

class ChannelVoiceChatData(BaseModel):
    voice_chat_id: Optional[str] = None
    state: Optional[str] = None
    join_muted: Optional[bool] = None
    participant_count: Optional[int] = None
    title: Optional[str] = None
    version: Optional[int] = None

class GroupVoiceChatUpdate(BaseModel):
    voice_chat_id: Optional[str] = None
    group_guid: Optional[str] = None
    action: Optional[str] = None
    group_voice_chat: Optional[GroupVoiceChatData] = None
    updated_parameters: Optional[List[str]] = None
    timestamp: Optional[str] = None
    chat_guid_type: Optional[ChatGuidType] = None

class ChannelVoiceChatUpdate(BaseModel):
    voice_chat_id: Optional[str] = None
    channel_guid: Optional[str] = None
    action: Optional[str] = None
    channel_voice_chat: Optional[ChannelVoiceChatData] = None
    updated_parameters: Optional[List[str]] = None
    timestamp: Optional[str] = None
    chat_guid_type: Optional[ChatGuidType] = None

class ExistGroupVoiceChat(BaseModel):
    voice_chat_id: Optional[str] = None
    state: Optional[str] = None
    join_muted: Optional[bool] = None
    participant_count: Optional[int] = None
    title: Optional[str] = None
    version: Optional[int] = None

class ExistChannelVoiceChat(BaseModel):
    voice_chat_id: Optional[str] = None
    state: Optional[str] = None
    join_muted: Optional[bool] = None
    participant_count: Optional[int] = None
    title: Optional[str] = None
    version: Optional[int] = None

class CreateVoiceCall(BaseModel):
    status: Optional[str] = None
    chat_update: Optional[ChatUpdate] = None
    message_update: Optional[MessageUpdate] = None
    group_voice_chat_update: Optional[GroupVoiceChatUpdate] = None
    channel_voice_chat_update: Optional[ChannelVoiceChatUpdate] = None
    exist_group_voice_chat: Optional[ExistGroupVoiceChat] = None
    exist_channel_voice_chat: Optional[ExistChannelVoiceChat] = None
    _client: Optional[str] = None
    original_update: Optional[str] = None



from typing import Optional, Literal
from pydantic import BaseModel
from .message import Message
from .payment_status import PaymentStatus


class Update(BaseModel):
    type: Literal['UpdatedMessage', 'NewMessage', 'RemovedMessage', 'StartedBot', 'StoppedBot', 'UpdatedPayment']
    chat_id: str
    removed_message_id: Optional[str]
    new_message: Message
    updated_message: Optional[Message]
    updated_payment: Optional[PaymentStatus]


import rubpy

class CheckChannelUsername:
    async def check_channel_username(
            self: "rubpy.Client",
            username: str,
    ):
        return await self.builder(
            name='checkChannelUsername',
            input={'username': username.replace('@', '')}
        )


from typing import Union


class GetMessagesInterval:
    async def get_messages_interval(
            self,
            object_guid: str,
            middle_message_id: Union[int, str],
    ):
        return await self.builder('getMessagesInterval',
                                  input={
                                      'object_guid': object_guid,
                                      'middle_message_id': str(middle_message_id),
                                  })


import rubpy
from typing import Union


class AddChannel:
    async def add_channel(
            self: "rubpy.Client",
            title: str,
            description: str = None,
            member_guids: Union[str, list] = None,
    ):
        input = {'description': description,
                 'title': title}

        if member_guids is not None:
            if isinstance(member_guids, str):
                member_guids = [member_guids]
            input['member_guids'] = member_guids

        return await self.builder('addChannel',
                                  input=input)


class LeaveGroupVoiceChat:
    async def leave_group_voice_chat(
            self,
            group_guid: str,
            voice_chat_id: str,
    ):
        return await self.builder('leaveGroupVoiceChat',
                                  input={
                                      'group_guid': group_guid,
                                      'voice_chat_id': voice_chat_id,
                                  })


class GetAvatars:
    async def get_avatars(
            self,
            object_guid: str,
    ):
        return await self.builder('getAvatars',
                                  input={
                                      'object_guid': object_guid,
                                  })


from typing import Union


class DeleteChatHistory:
    async def delete_chat_history(
            self,
            object_guid: str,
            last_message_id: Union[str, int],
    ):
        return await self.builder('deleteChatHistory',
                                  input={
                                      'object_guid': object_guid,
                                      'last_message_id': str(last_message_id),
                                  })


import re

MARKDOWN_RE = (
    r'\*\*(.*?)\*\*|`(.*?)`|__(.*?)__|--(.*?)--|~~(.*?)~~|\|\|(.*?)\|\||\[(.*?)\]\((\S+)\)'
)

class Markdown:
    def __init__(self) -> None:
        self.pattern = re.compile(MARKDOWN_RE)

    def to_metadata(self, text: str) -> dict:
        meta_data_parts = []

        while True:
            for markdown in self.pattern.finditer(text):
                string = markdown.group(0)
                span = markdown.span()
                from_index = span[0]

                if string.startswith('**'):
                    text = self.pattern.sub(r'\1', text, count=1)
                    meta_data_parts.append({
                        'type': 'Bold',
                        'from_index': from_index,
                        'length': len(string) - 4,
                    })
                    break
                elif string.startswith('`'):
                    text = self.pattern.sub(r'\2', text, count=1)
                    meta_data_parts.append({
                        'type': 'Mono',
                        'from_index': from_index,
                        'length': len(string) - 4,
                    })
                    break
                elif string.startswith('__'):
                    text = self.pattern.sub(r'\3', text, count=1)
                    meta_data_parts.append({
                        'type': 'Italic',
                        'from_index': from_index,
                        'length': len(string) - 4,
                    })
                    break
                elif string.startswith('--'):
                    text = self.pattern.sub(r'\4', text, count=1)
                    meta_data_parts.append({
                        'type': 'Underline',
                        'from_index': from_index,
                        'length': len(string) - 4,
                    })
                    break
                elif string.startswith('~~'):
                    text = self.pattern.sub(r'\5', text, count=1)
                    meta_data_parts.append({
                        'type': 'Strike',
                        'from_index': from_index,
                        'length': len(string) - 4,
                    })
                    break
                elif string.startswith('||'):
                    text = self.pattern.sub(r'\6', text, count=1)
                    meta_data_parts.append({
                        'type': 'Spoiler',
                        'from_index': from_index,
                        'length': len(string) - 4,
                    })
                    break

                else:
                    text = self.pattern.sub(r'\7', text, count=1)
                    mention_text_object_type = 'hyperlink'
                    mention_type = 'MentionText'
                    mention_text_object_guid = markdown.group(8)
                    length = len(markdown.group(7))

                    if mention_text_object_guid.startswith('u'):
                        mention_text_object_type = 'User'
                    elif mention_text_object_guid.startswith('g'):
                        mention_text_object_type = 'Group'
                    elif mention_text_object_guid.startswith('c'):
                        mention_text_object_type = 'Channel'
                    else:
                        mention_type = 'Link'

                    if mention_type == 'MentionText':
                        meta_data_parts.append({
                            'type': mention_type,
                            'from_index': from_index,
                            'length': length,
                            'mention_text_object_guid': mention_text_object_guid,
                            'mention_text_object_type': mention_text_object_type
                        })
                        break

                    else:
                        meta_data_parts.append({
                            'from_index': from_index,
                            'length': length,
                            'link': {
                                'hyperlink_data': {
                                    'url': mention_text_object_guid,
                                },
                                'type': mention_text_object_type,
                            },
                            'type': mention_type,
                        })
                        break

            else:
                break

        result = {'text': text.strip()}

        if meta_data_parts:
            result['metadata'] = {'meta_data_parts': meta_data_parts}

        return result


import rubpy

class ActionOnStickerSet:
    async def action_on_sticker_set(
            self: "rubpy.Client",
            sticker_set_id: str,
            action: str = 'Add',
    ):
        if action not in ('Add', 'Remove'):
            raise ValueError('The `action` argument can only be in `("Add", "Remove")`.')

        return await self.builder(name='actionOnStickerSet',
                                  input={
                                      'sticker_set_id': sticker_set_id,
                                      'action': action,
                                  })


from .add_group import AddGroup
from .add_group_members import AddGroupMembers
from .ban_group_member import BanGroupMember
from .create_group_voice_chat import CreateGroupVoiceChat
from .delete_no_access_group_chat import DeleteNoAccessGroupChat
from .edit_group_info import EditGroupInfo
from .get_banned_group_members import GetBannedGroupMembers
from .get_group_admin_access_list import GetGroupAdminAccessList
from .get_group_admin_members import GetGroupAdminMembers
from .get_group_all_members import GetGroupAllMembers
from .get_group_default_access import GetGroupDefaultAccess
from .get_group_info import GetGroupInfo
from .get_group_link import GetGroupLink
from .get_group_mention_list import GetGroupMentionList
from .get_group_voice_chat_updates import GetGroupVoiceChatUpdates
from .group_preview_by_join_link import GroupPreviewByJoinLink
from .join_group import JoinGroup
from .leave_group import LeaveGroup
from .leave_group_voice_chat import LeaveGroupVoiceChat
from .remove_group import RemoveGroup
from .set_group_admin import SetGroupAdmin
from .set_group_default_access import SetGroupDefaultAccess
from .set_group_link import SetGroupLink
from .set_group_voice_chat_setting import SetGroupVoiceChatSetting


class Groups(
    AddGroup,
    AddGroupMembers,
    BanGroupMember,
    CreateGroupVoiceChat,
    DeleteNoAccessGroupChat,
    EditGroupInfo,
    GetBannedGroupMembers,
    GetGroupAdminAccessList,
    GetGroupAdminMembers,
    GetGroupAllMembers,
    GetGroupDefaultAccess,
    GetGroupInfo,
    GetGroupLink,
    GetGroupMentionList,
    GetGroupVoiceChatUpdates,
    GroupPreviewByJoinLink,
    JoinGroup,
    LeaveGroup,
    LeaveGroupVoiceChat,
    RemoveGroup,
    SetGroupAdmin,
    SetGroupDefaultAccess,
    SetGroupLink,
    SetGroupVoiceChatSetting,
):
    pass